-- reaperspawn.lua for SRB2Kart
-- makes players respawn at reaper waypoints instead of their usual respawn positions
-- thing number can be changed by setting the reaperspawnthingnum map header value
-- by minenice

local version = 1
if reaperspawn and reaperspawn.version and reaperspawn.version >= version then return end
rawset(_G, "reaperspawn", {})

rawset(_G, "reaperspawn_points", {})
addHook("NetVars", function(net)
    reaperspawn_points = net(reaperspawn_points)
end)

function reaperspawn.ReaperSpawnData()
    return {
        lastStarPost = -1
    }
end

function reaperspawn.SetPlayerRespawn(p, x, y, z, angle)
    p.starpostx = x
    p.starposty = y
    p.starpostz = z
    p.starpostangle = angle
end

function reaperspawn.PlayerPostThink(p)
    if reaperspawn_points == nil then return end
    if reaperspawn_points == {} then return end
    if not mapheaderinfo[gamemap].reaperspawn then return end
    if leveltime == 0 then return end

    if p.reaperSpawnData == nil then
        p.reaperSpawnData = reaperspawn.ReaperSpawnData()
    end

    if p.starpostnum ~= p.reaperSpawnData.lastStarPost then
        p.reaperSpawnData.lastStarPost = p.starpostnum
        -- use normal behaviour if a reaper point is not set for this starpost num (or for starpost 0)
        if reaperspawn_points[p.starpostnum] == nil then
            return
        end
        local point = reaperspawn_points[p.starpostnum]
        reaperspawn.SetPlayerRespawn(p, point.x, point.y, point.z, point.angle)
    end
end

function reaperspawn.GetReaperPoints()
    if not mapheaderinfo[gamemap].reaperspawn then return end

    local thingnum = 3199
    if mapheaderinfo[gamemap].reaperspawnthingnum and tonumber(mapheaderinfo[gamemap].reaperspawnthingnum) then
        thingnum = tonumber(mapheaderinfo[gamemap].reaperspawnthingnum)
        -- print("Thing ID used for Reaperspawn points is now "..thingnum)
    end

    reaperspawn_points = {}
    for thing in mapthings.iterate do
        if thing.type ~= thingnum then continue end
        local x = thing.x
        local y = thing.y

        local sector = R_PointInSubsector(x * FRACUNIT, y * FRACUNIT).sector
        local flip = thing.options & 2 == 2 and 1 or 0
        local thingZ, z
        if flip then
            if sector.c_slope then
                thingZ = P_GetZAt(sector.c_slope, x * FRACUNIT, y * FRACUNIT)
            else
                thingZ = sector.ceilingheight
            end
            z = thingZ/FRACUNIT - (thing.options>>4)
        else
            if sector.f_slope then
                thingZ = P_GetZAt(sector.f_slope, x * FRACUNIT, y * FRACUNIT)
            else
                thingZ = sector.floorheight
            end
            z = thingZ/FRACUNIT + (thing.options>>4)
        end
        local angle = thing.angle % 360*ANG1
        local num = thing.angle / 360*ANG1 / ANG1
        
        -- debug
        -- print("Reaper point at ("..x..", "..y..", "..z..") with angle "..angle/ANG1.." and num "..num.." and flip "..flip)

        -- count from zero but starposts start at one
        -- only one repearspawn point per starpost (no support for split paths, maybe eventually idk this game's distance calculation system really doesn't like em to begin with)
        if reaperspawn_points[num + 1] then
            print("\x82WARNING:\x80 Reaperspawn point "..num.." already exists and is being overwritten.")
        end
        reaperspawn_points[num + 1] = {x = x, y = y, z = z, angle = angle, flip = flip}
    end

    for i = 0, #players-1 do
        if players[i] then
            players[i].reaperSpawnData = nil
        end
    end
end

if reaperspawn.version == nil then
    addHook("PostThinkFrame", do
        for i = 0, #players-1 do
            if players[i] then
                reaperspawn.PlayerPostThink(players[i])
            end
        end
    end)

    addHook("MapLoad", do
        reaperspawn.GetReaperPoints()
    end)
end
reaperspawn.version = version